package dnanalytics.view.tools.align;

import dnanalytics.utils.FileManager;
import dnanalytics.utils.Settings;
import dnanalytics.view.DNAMain;
import dnanalytics.view.tools.Tool;
import dnanalytics.worker.Aligner;
import dnanalytics.worker.Worker;
import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.ResourceBundle;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.scene.Node;
import javafx.scene.control.CheckBox;
import javafx.scene.control.RadioButton;
import javafx.scene.control.TextField;
import javafx.scene.control.ToggleGroup;

public class AlignView implements Tool {

    @FXML
    private static ResourceBundle resources = DNAMain.getResources();
    @FXML
    private URL location;
    @FXML
    private static TextField dbsnp;
    @FXML
    private static TextField forward;
    @FXML
    private static TextField mills;
    @FXML
    private static TextField output;
    @FXML
    private static TextField phase1;
    @FXML
    private static CheckBox reduce;
    @FXML
    private static TextField reverse;
    @FXML
    private static ToggleGroup x1;

    @FXML
    void initialize() {
        resources = DNAMain.getResources();
        // Recover settings
        String value = Settings.getProperty("align.dbsnp");
        dbsnp.setText(value != null ? value : "");
        value = Settings.getProperty("align.mills");
        mills.setText(value != null ? value : "");
        value = Settings.getProperty("align.phase1");
        phase1.setText(value != null ? value : "");
    }

    @FXML
    void selectForward(ActionEvent event) {
        FileManager.setOpenFile(FileManager.FASTQ_DESCRIPTION,
                FileManager.FASTQ_DESCRIPTION, FileManager.FASTQ_FILTERS, forward);
    }

    @FXML
    void selectReverse(ActionEvent event) {
        FileManager.setOpenFile(FileManager.FASTQ_DESCRIPTION,
                FileManager.FASTQ_DESCRIPTION, FileManager.FASTQ_FILTERS, reverse);
    }

    @FXML
    void selectDbsnp(ActionEvent event) {
        File f = FileManager.openVCF(dbsnp);
        if (f != null) {
            Settings.setProperty("align.dbsnp", f.getAbsolutePath());
        }
    }

    @FXML
    void selectMills(ActionEvent event) {
        File f = FileManager.openVCF(mills);
        if (f != null) {
            Settings.setProperty("align.mills", f.getAbsolutePath());
        }
    }

    @FXML
    void selectPhase1(ActionEvent event) {
        File f = FileManager.openVCF(phase1);
        if (f != null) {
            Settings.setProperty("align.phase1", f.getAbsolutePath());
        }
    }

    @FXML
    void selectOutput(ActionEvent event) {
        FileManager.setSaveFile(FileManager.SAM_BAM_DESCRIPTION,
                FileManager.SAM_BAM_DESCRIPTION, FileManager.SAM_BAM_FILTERS,
                FileManager.BAM_EXTENSION, output);
    }

    @Override
    public Node getView() {
        try {
            return FXMLLoader.load(getClass().getResource("AlignView.fxml"),
                    DNAMain.getResources());
        } catch (IOException ex) {
            return null;
        }
    }

    @Override
    public Worker getWorker() {
        // Check for all params
        if (Settings.getGenome() == null || !Settings.getGenome().exists()) {
            System.err.println(resources.getString("no.genome"));
            return null;
        }
        if (!new File(Settings.getProperty("tempDir")).exists()) {
            System.err.println(resources.getString("no.temp"));
            return null;
        }
        if (!new File(forward.getText()).exists()) {
            System.err.println(resources.getString("no.forward"));
            return null;
        }
        if (!new File(reverse.getText()).exists()) {
            System.err.println(resources.getString("no.reverse"));
            return null;
        }
        if (!new File(mills.getText()).exists()) {
            System.err.println(resources.getString("no.mills"));
            return null;
        }
        if (!new File(phase1.getText()).exists()) {
            System.err.println(resources.getString("no.phase1"));
            return null;
        }
        if (!new File(dbsnp.getText()).exists()) {
            System.err.println(resources.getString("no.dbsnp"));
            return null;
        }
        if (output.getText().isEmpty()) {
            System.err.println(resources.getString("no.output"));
            return null;
        }
        RadioButton button = (RadioButton) x1.getSelectedToggle();
        // Everything seems legal. Launch Worker.
        return new Aligner(Settings.getProperty("tempDir"),
                forward.getText(), reverse.getText(), Settings.getGenome().
                getAbsolutePath(), dbsnp.getText(), mills.getText(), phase1.
                getText(), output.getText(), button.getId().equals("isIllumina"),
                reduce.isSelected());
    }

    @Override
    public String getTitle() {
        return resources.getString("align.title");
    }

    @Override
    public String getStyleID() {
        return "alignb";
    }
}
