package dnanalytics.view.tools.combineVariants;

import dnanalytics.utils.FileManager;
import dnanalytics.utils.Settings;
import dnanalytics.worker.WorkerScript;
import dnanalytics.view.DNAMain;
import dnanalytics.view.tools.Tool;
import dnanalytics.worker.Worker;
import java.io.File;
import java.io.IOException;
import java.util.ResourceBundle;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.scene.Node;
import javafx.scene.control.ListView;
import javafx.scene.control.TextField;
import javafx.scene.control.ToggleButton;
import javafx.scene.control.ToggleGroup;

/**
 * FXML Controller class
 * <p/>
 * @author uai
 */
public class CombineVariants implements Tool {

    @FXML
    private static ToggleGroup combineGroup;
    @FXML
    private static TextField combinedVCF;
    @FXML
    private static ListView<String> vcfList;
    private static ResourceBundle resources = DNAMain.getResources();

    @FXML
    void addVCF(ActionEvent event) {
        File file = FileManager.selectFile("Select VCF file",
                FileManager.VCF_DESCRIPTION, FileManager.VCF_FILTERS);
        if (file != null) {
            vcfList.getItems().add(file.getAbsolutePath());
        }
    }

    @FXML
    void deleteVCF() {
        try {
            vcfList.getItems().
                    remove(vcfList.getSelectionModel().getSelectedIndex());
        } catch (ArrayIndexOutOfBoundsException ex) {
            System.err.println("Select a file in the list");
        }
    }

    @FXML
    void selectOutput(ActionEvent event) {
        FileManager.saveVCF(combinedVCF);
    }

    @Override
    public Node getView() {
        try {
            return FXMLLoader.load(getClass().
                    getResource("CombineVariants.fxml"),
                    DNAMain.getResources());
        } catch (IOException ex) {
            return null;
        }
    }

    private boolean checkParameters() {
        if (Settings.getGenome() == null) {
            System.err.println(DNAMain.getResources().getString("no.genome"));
            return false;
        }
        if (combinedVCF.getText().isEmpty()) {
            System.err.println(DNAMain.getResources().getString("no.output"));
            return false;
        }
        return true;
    }

    @Override
    public Worker getWorker() {

        if (!checkParameters()) {
            return null;
        }
        return new WorkerScript() {
            String variants = "";
            String gatk = "";

            private void initializeSettings() {
                for (String var : vcfList.getItems()) {
                    variants += " -V " + var;
                }
                gatk = "java -jar software" + File.separator + "gatk" + File.separator
                        + "GenomeAnalysisTK.jar";

            }

            private void intersectVariants() {
                executeCommand(gatk
                        + " -T CombineVariants"
                        + " -R " + Settings.getGenome()
                        + " -minN " + vcfList.getItems().size()
                        + " -o " + combinedVCF.getText()
                        + variants);

            }

            private void aggregateVariants() {
                executeCommand(gatk
                        + " -T CombineVariants"
                        + " -R " + Settings.getGenome()
                        + " -o " + combinedVCF.getText()
                        + variants);
            }

            private void deductVariants() {
                executeCommand(gatk
                        + " -T SelectVariants"
                        + " -R " + Settings.getGenome()
                        + " -V " + vcfList.getItems().get(0)
                        + " -o " + combinedVCF.getText()
                        + " --discordance " + vcfList.getItems().get(1));
            }

            @Override
            protected int start() {
                initializeSettings();
                updateTitle("combinig " + new File(combinedVCF.getText()).getName());

                switch (((ToggleButton) combineGroup.getSelectedToggle()).getId()) {
                    case "intersection":
                        intersectVariants();
                        break;
                    case "aggregation":
                        aggregateVariants();
                        break;
                    case "difference":
                        deductVariants();
                }
                return 0;
            }
        };
    }

    @Override
    public String getTitle() {
        return resources.getString("combine.title");
    }

    @Override
    public String getStyleID() {
        return "combineVariants";
    }
}
