package dnanalytics.view.tools.filterFrequencies;

import dnanalytics.utils.FileManager;
import dnanalytics.worker.WorkerScript;
import dnanalytics.view.DNAMain;
import dnanalytics.view.tools.Tool;
import dnanalytics.worker.Worker;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ResourceBundle;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import javafx.beans.value.ChangeListener;
import javafx.beans.value.ObservableValue;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.scene.Node;
import javafx.scene.control.Label;
import javafx.scene.control.TextField;

public class FilterFrequencies implements Tool {

    @FXML
    private static ResourceBundle resources = DNAMain.getResources();
    @FXML
    private static TextField frequencyFile;
    @FXML
    private static TextField columnField;
    @FXML
    private static TextField maxFrequency;
    @FXML
    private static Label columnName;

    public void initialize() {
        columnField.textProperty().addListener(new ChangeListener<String>() {
            @Override
            public void changed(ObservableValue<? extends String> ov, String t,
                    String t1) {
                columnName.setText("Check first line");
                if (new File(frequencyFile.getText()).exists()) {
                    BufferedReader br = null;
                    try {
                        br = new BufferedReader(new FileReader(frequencyFile.
                                getText()));
                        String line = br.readLine();
                        if (line != null) {
                            String[] columns = line.split("\t");
                            int column = Integer.valueOf(columnField.getText());
                            if (column <= columns.length && column > 0) {
                                columnName.setText(columns[column - 1]);
                            }
                        }
                    } catch (NumberFormatException ex) {
                    } catch (FileNotFoundException ex) {
                        Logger.getLogger(DNAMain.class.getName()).log(
                                Level.SEVERE, null, ex);
                    } catch (IOException ex) {
                        Logger.getLogger(DNAMain.class.getName()).log(
                                Level.SEVERE, null, ex);
                    } finally {
                        try {
                            if (br != null) {
                                br.close();
                            }
                        } catch (IOException ex) {
                            Logger.getLogger(DNAMain.class.getName()).log(
                                    Level.SEVERE, null, ex);
                        }
                    }
                }
            }
        });

    }

    @FXML
    void selectFrequencyFile(ActionEvent event) {
        FileManager.setOpenFile(FileManager.TSV_DESCRIPTION, FileManager.TSV_DESCRIPTION,
                FileManager.TSV_FILTERS, frequencyFile);
    }

    @Override
    public Node getView() {
        try {
            return FXMLLoader.load(getClass().getResource(
                    "FilterFrequencies.fxml"), DNAMain.getResources());
        } catch (IOException ex) {
            return null;
        }
    }

    @Override
    public Worker getWorker() {
        if (frequencyFile.getText().isEmpty()) {
            System.err.println(resources.getString("no.output"));
            return null;
        }
        final int column;
        try {
            column = Integer.valueOf(columnField.getText());
        } catch (NumberFormatException ex) {
            System.err.println("Wrong column number format");
            return null;
        }
        final double freq;
        try {
            freq = Double.valueOf(maxFrequency.getText());
        } catch (NumberFormatException ex) {
            System.err.println("Wrong frequency number format");
            return null;
        }
        return new WorkerScript() {
            @Override
            protected int start() {
                String line;
                BufferedReader br = null;
                BufferedWriter bw = null;
                String[] columns;

                updateTitle("Filtering "
                        + new File(frequencyFile.getText()).getName());
                try {
                    Thread.sleep(3000);
                } catch (InterruptedException ex) {
                    Logger.getLogger(FilterFrequencies.class.getName()).log(Level.SEVERE,
                            null, ex);
                }
                try {
                    File fr = new File(frequencyFile.getText());
                    br = new BufferedReader(new FileReader(fr));
                    bw = new BufferedWriter(new FileWriter(frequencyFile.
                            getText().replace(".tsv", "_filtered.tsv")));
                    long total = fr.length();
                    long read = 0;
                    updateProgress(read, total);
                    // Looking for column title
                    if ((line = br.readLine()) != null) {
                        columns = line.split("\t");
                        if (column <= columns.length && column > 0) {
                            outStream.println(
                                    "Looking at " + columns[column - 1]);
                        }
                        bw.write(line);
                        bw.newLine();
                    }
                    // Here we go, give me one line
                    while ((line = br.readLine()) != null) {
                        // Nice progess bar view
                        updateProgress(read, total);
                        read += line.length();

                        // So, you have partitioned it in 'tab' separated fields?
                        columns = line.split("\t");
                        // Don't lie to me (told me column 6), are there at least 6 columns?
                        if (columns.length >= column) {
                            Pattern pattern = Pattern.compile("-?[\\d\\.]+");
                            // Oh oh, this field is empty, i will leave it as it was.
                            if (columns[column - 1].isEmpty()) {
                                bw.write(line);
                                bw.newLine();
                                continue;
                            }
                            // You told me column 6? This is 5 for a computer
                            Matcher m = pattern.matcher(columns[column - 1]);
                            // Ok, are these really a number?
                            while (m.find()) {
                                try {
                                    double d = Double.valueOf(m.group());
                                    // Oh god, is any of this frequencies less than your maximum frequency?
                                    if (d <= freq) {
                                        // Yes? Write line and get out of here, NEXT LINE!!!!
                                        bw.write(line);
                                        bw.newLine();
                                        break;
                                    }
                                } catch (NumberFormatException ex) {
                                }
                            }
                        }
                    }
                } catch (FileNotFoundException ex) {
                    errStream.println(
                            "File " + frequencyFile.getText() + "not found");
                } catch (IOException ex) {
                    System.err.println("Error reading file");
                } finally {
                    try {
                        if (br != null) {
                            br.close();
                        }
                        if (bw != null) {
                            bw.close();
                        }
                    } catch (IOException ex) {
                        Logger.getLogger(DNAMain.class.getName()).log(
                                Level.SEVERE, null, ex);
                    }
                }
                return 0;
            }

        };
    }

    @Override
    public String getTitle() {
        return resources.getString("filter.title");
    }

    @Override
    public String getStyleID() {
        return "filterFrequencies";
    }
}
