package dnanalytics.view.tools.selectVariants;

import dnanalytics.utils.FileManager;
import dnanalytics.worker.WorkerScript;
import dnanalytics.utils.Settings;
import dnanalytics.view.DNAMain;
import dnanalytics.view.tools.Tool;
import dnanalytics.worker.Worker;
import java.io.File;
import java.io.IOException;
import java.util.ResourceBundle;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.scene.Node;
import javafx.scene.control.TextField;

public class SelectVariants implements Tool {

    @FXML
    private static ResourceBundle resources = DNAMain.getResources();
    @FXML
    private static TextField expression;
    @FXML
    private static TextField input;
    @FXML
    private static TextField output;

    @FXML
    void selectVariantsInput(ActionEvent event) {
        FileManager.openVCF(input);
    }

    @FXML
    void selectVariantsOutput(ActionEvent event) {
        FileManager.saveVCF(output);
    }

    @Override
    public Node getView() {
        try {
            return FXMLLoader.
                    load(getClass().getResource("SelectVariants.fxml"),
                    DNAMain.getResources());
        } catch (IOException ex) {
            return null;
        }
    }

    @Override
    public Worker getWorker() {
        // Check the params in the main thread, to avoid launching a dummy Worker.
        if (Settings.getGenome() == null) {
            System.err.println(resources.getString("no.genome"));
            return null;
        }
        if (!new File(input.getText()).exists()) {
            System.err.println(resources.getString("no.input"));
            return null;
        }
        if (output.getText().isEmpty()) {
            System.err.println(resources.getString("no.output"));
            return null;
        }
        if (expression.getText().isEmpty()) {
            System.err.println(resources.getString("no.expression"));
            return null;
        }

        // The Worker selecter is created, it will select the variants in background
        return new WorkerScript() {
            @Override
            protected int start() {
                updateTitle("Selecting " + new File(input.getText()).getName());
                updateMessage(resources.getString("select.select"));
                executeCommand(
                        "java -jar software" + File.separator + "gatk" + File.separator + "GenomeAnalysisTK.jar"
                        + " -T SelectVariants"
                        + " -R " + Settings.getGenome()
                        + " -V " + input.getText()
                        + " -restrictAllelesTo BIALLELIC"
                        + " -o " + output.getText()
                        + " -select \"" + expression.getText() + '\"');
                return 0;
            }
        };
    }

    @Override
    public String getTitle() {
        return resources.getString("select.title");
    }

    @Override
    public String getStyleID() {
        return "selectVariants";
    }
}
